"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Ipld = require('ipld');
/**
 * Underlies a ChainTree, it represents a DAG of IPLD nodes and supports resolving accross
 * multiple nodes.
 * @public
 */
class Dag {
    constructor(tip, store) {
        this.tip = tip;
        this.dagStore = new Ipld({ blockService: store });
    }
    /**
     * Gets a node from the dag
     * @param cid - The CID of the node to get from the DAG
     * @public
     */
    get(cid) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.dagStore.get(cid);
        });
    }
    /**
     *
     * @param path - a path to the desired node/key in the DAG (eg /path/to/data). Array form (eg ['path', 'to', 'data']) is deprecated
     * @public
     */
    resolve(path) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.resolveAt(this.tip, path);
        });
    }
    /**
     * Similar to resolve, but allows you to start at a specific tip of a dag rather than the current tip.
     * @param tip - The tip of the dag to start at
     * @param path - the path to find the value. Array form is deprecated, use string form (eg /path/to/data) instead
     * @public
     */
    resolveAt(tip, path) {
        return __awaiter(this, void 0, void 0, function* () {
            let strPath;
            if (isArray(path)) {
                console.warn('passing in arrays to resolve is deprecated, use the string form (eg /path/to/data) instead');
                strPath = path.join("/");
            }
            else {
                strPath = path;
            }
            const resolved = this.dagStore.resolve(tip, strPath);
            let lastVal;
            try {
                lastVal = yield resolved.last();
            }
            catch (err) {
                const e = err;
                if (!e.message.startsWith("Object has no property")) {
                    throw err;
                }
            }
            // nothing was resolvable, return full path as the remainder
            if (typeof lastVal === 'undefined') {
                return { remainderPath: strPath.split('/'), value: null };
            }
            // if remainderPath is not empty, then the value was not found and an
            // error was thrown on the second iteration above - use the remainderPath
            // from the first iteration, but return nil for the error
            if (lastVal.remainderPath != '') {
                return { remainderPath: lastVal.remainderPath.split('/'), value: null };
            }
            return { remainderPath: [], value: lastVal.value };
        });
    }
}
exports.Dag = Dag;
function isArray(path) {
    return path.join !== undefined;
}
