/**
 * EcdsaKey defines the public/private key-pairs used to interact with Tupelo.
 * It also supportes generating new keys either randomly or through a passphrase.
 * @public
 */
export declare class EcdsaKey {
    privateKey?: Uint8Array;
    publicKey: Uint8Array;
    /**
     * Generate a new keypair with random bits.
     * @public
     */
    static generate: () => Promise<EcdsaKey>;
    /**
     * Generate a new key based on a passphrase and salt (this goes through the Warp wallet treatment {@link https://keybase.io/warp/warp_1.0.9_SHA256_a2067491ab582bde779f4505055807c2479354633a2216b22cf1e92d1a6e4a87.html})
     */
    static passPhraseKey: (phrase: Uint8Array, salt: Uint8Array) => Promise<EcdsaKey>;
    static fromBytes: (bytes: Uint8Array) => Promise<EcdsaKey>;
    constructor(publicKeyBits: Uint8Array, privateKeyBits?: Uint8Array);
    /**
     * Returns the address of the public key (ethereum address format)
     * @public
     */
    address(): Promise<string>;
    /**
     * Returns the DID generated from the public key address
     * @public
     */
    toDid(): Promise<string>;
    /**
     * deprecated use address() or toDid()
     * @deprecated
     */
    keyAddr(): Promise<string>;
}
//# sourceMappingURL=crypto.d.ts.map