"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const tupelo_1 = require("./tupelo");
/**
 * EcdsaKey defines the public/private key-pairs used to interact with Tupelo.
 * It also supportes generating new keys either randomly or through a passphrase.
 * @public
 */
class EcdsaKey {
    constructor(publicKeyBits, privateKeyBits) {
        this.publicKey = publicKeyBits;
        this.privateKey = privateKeyBits;
    }
    /**
     * Returns the address of the public key (ethereum address format)
     * @public
     */
    address() {
        return __awaiter(this, void 0, void 0, function* () {
            return tupelo_1.Tupelo.ecdsaPubkeyToAddress(this.publicKey);
        });
    }
    /**
     * Returns the DID generated from the public key address
     * @public
     */
    toDid() {
        return __awaiter(this, void 0, void 0, function* () {
            return tupelo_1.Tupelo.ecdsaPubkeyToDid(this.publicKey);
        });
    }
    /**
     * deprecated use address() or toDid()
     * @deprecated
     */
    keyAddr() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.toDid();
        });
    }
}
exports.EcdsaKey = EcdsaKey;
/**
 * Generate a new keypair with random bits.
 * @public
 */
EcdsaKey.generate = () => __awaiter(void 0, void 0, void 0, function* () {
    const pair = yield tupelo_1.Tupelo.generateKey();
    const key = new EcdsaKey(pair[1], pair[0]);
    return key;
});
/**
 * Generate a new key based on a passphrase and salt (this goes through the Warp wallet treatment {@link https://keybase.io/warp/warp_1.0.9_SHA256_a2067491ab582bde779f4505055807c2479354633a2216b22cf1e92d1a6e4a87.html})
 */
EcdsaKey.passPhraseKey = (phrase, salt) => __awaiter(void 0, void 0, void 0, function* () {
    const pair = yield tupelo_1.Tupelo.passPhraseKey(phrase, salt);
    return new EcdsaKey(pair[1], pair[0]);
});
EcdsaKey.fromBytes = (bytes) => __awaiter(void 0, void 0, void 0, function* () {
    const pair = yield tupelo_1.Tupelo.keyFromPrivateBytes(bytes);
    return new EcdsaKey(pair[1], pair[0]);
});
