import CID from 'cids';
import { Transaction } from 'tupelo-messages';
import { TokenPayload } from 'tupelo-messages/transactions/transactions_pb';
import { IBlockService } from './chaintree/dag/dag';
import ChainTree from './chaintree/chaintree';
import { Proof } from 'tupelo-messages/gossip/gossip_pb';
import { NotaryGroup } from 'tupelo-messages/config/config_pb';
/**
 * The interface describing libp2p pubsub
 * @public
 */
export interface IPubSub {
    publish(topic: string, data: Uint8Array, cb: Function): null;
    subscribe(topic: string, onMsg: Function, cb: Function): null;
    unsubscribe(topic: string, onMsg: Function, cb: Function): null;
}
interface ITransactionPayloadOpts {
    blockService: IBlockService;
    tip: CID;
    tokenName: string;
    sendId: string;
    proof: Proof;
}
/**
 * Tupelo is the more "raw" namespace, it is generally expected
 * that you would use higher level wrapper classes around this namespace.
 * For example: {@link EcdsaKey} or {@link Community#"class"}
 * @public
 */
export declare namespace Tupelo {
    function generateKey(): Promise<Uint8Array[]>;
    function passPhraseKey(phrase: Uint8Array, salt: Uint8Array): Promise<Uint8Array[]>;
    function keyFromPrivateBytes(bytes: Uint8Array): Promise<Uint8Array[]>;
    function ecdsaPubkeyToDid(pubKey: Uint8Array): Promise<string>;
    function ecdsaPubkeyToAddress(pubKey: Uint8Array): Promise<string>;
    function getTip(did: string): Promise<Proof>;
    /**
     * getLatest returns a proof (like getTip)
     * but before resolving it has loaded up all the new
     * blocks for the tree.
     * @param did - the did of the tree
     * @internal
     */
    function getLatest(did: string): Promise<ChainTree>;
    function newEmptyTree(store: IBlockService, publicKey: Uint8Array): Promise<CID>;
    function tokenPayloadForTransaction(opts: ITransactionPayloadOpts): Promise<TokenPayload>;
    /**
     * verifyProof takes a proof and makes sure it meets the requirements
     * for being a valid signed proof (2/3 of signers, valid signature, etc)
     * @param proof - the Proof (often returned by a playTransactions)
     * @public
     */
    function verifyProof(proof: Proof): Promise<boolean>;
    function startClient(pubsub: IPubSub, group: NotaryGroup, store: IBlockService): Promise<void>;
    function playTransactions(tree: ChainTree, transactions: Transaction[]): Promise<Proof>;
    function setLogLevel(name: string, level: string): Promise<void>;
}
export default Tupelo;
//# sourceMappingURL=tupelo.d.ts.map