"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cids_1 = __importDefault(require("cids"));
const go = require('./js/go');
const transactions_pb_1 = require("tupelo-messages/transactions/transactions_pb");
const chaintree_1 = __importDefault(require("./chaintree/chaintree"));
const gossip_pb_1 = require("tupelo-messages/gossip/gossip_pb");
const debug_1 = __importDefault(require("debug"));
const logger = debug_1.default("tupelo");
class UnderlyingWasm {
    generateKey() {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    passPhraseKey(phrase, salt) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    keyFromPrivateBytes(bytes) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    ecdsaPubkeyToDid(pubKey) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    ecdsaPubkeyToAddress(pubKey) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    getTip(did) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    getLatest(did) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    newEmptyTree(store, publicKey) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    playTransactions(opts) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    startClient(opts) {
        // replaced by wasm
    }
    tokenPayloadForTransaction(opts) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    verifyProof(proofBytes) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    setLogLevel(name, level) {
        return; // replaced by wasm
    }
}
var TupeloWasm;
(function (TupeloWasm) {
    let _tupelowasm;
    TupeloWasm.get = () => {
        if (_tupelowasm !== undefined) {
            return _tupelowasm;
        }
        _tupelowasm = new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            const wasm = new UnderlyingWasm;
            logger("go.run for first time");
            go.run("./main.wasm");
            yield go.ready();
            go.populate(wasm, {
                "cids": cids_1.default,
                "ipfs-block": require('ipfs-block'),
            });
            resolve(wasm);
        }));
        return _tupelowasm;
    };
})(TupeloWasm || (TupeloWasm = {}));
/**
 * Tupelo is the more "raw" namespace, it is generally expected
 * that you would use higher level wrapper classes around this namespace.
 * For example: {@link EcdsaKey} or {@link Community#"class"}
 * @public
 */
var Tupelo;
(function (Tupelo) {
    let clientStore;
    // generateKey returns a two element array of the bytes for [privateKey, publicKey]
    function generateKey() {
        return __awaiter(this, void 0, void 0, function* () {
            logger("generateKey");
            const tw = yield TupeloWasm.get();
            return tw.generateKey();
        });
    }
    Tupelo.generateKey = generateKey;
    function passPhraseKey(phrase, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("passPhraseKey");
            const tw = yield TupeloWasm.get();
            return tw.passPhraseKey(phrase, salt);
        });
    }
    Tupelo.passPhraseKey = passPhraseKey;
    function keyFromPrivateBytes(bytes) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("keyFromPrivateBytes");
            const tw = yield TupeloWasm.get();
            return tw.keyFromPrivateBytes(bytes);
        });
    }
    Tupelo.keyFromPrivateBytes = keyFromPrivateBytes;
    function ecdsaPubkeyToDid(pubKey) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("ecdsaPubkeyToDid");
            const tw = yield TupeloWasm.get();
            return tw.ecdsaPubkeyToDid(pubKey);
        });
    }
    Tupelo.ecdsaPubkeyToDid = ecdsaPubkeyToDid;
    function ecdsaPubkeyToAddress(pubKey) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("ecdsaPubkeyToAddress");
            const tw = yield TupeloWasm.get();
            return tw.ecdsaPubkeyToAddress(pubKey);
        });
    }
    Tupelo.ecdsaPubkeyToAddress = ecdsaPubkeyToAddress;
    function getTip(did) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("getTip: ", did);
            const tw = yield TupeloWasm.get();
            try {
                let stateBits = yield tw.getTip(did);
                return gossip_pb_1.Proof.deserializeBinary(stateBits);
            }
            catch (err) {
                throw err;
            }
        });
    }
    Tupelo.getTip = getTip;
    /**
     * getLatest returns a proof (like getTip)
     * but before resolving it has loaded up all the new
     * blocks for the tree.
     * @param did - the did of the tree
     * @internal
     */
    function getLatest(did) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("getLatest: ", did);
            if (!clientStore) {
                throw new Error("you must start the client first");
            }
            const tw = yield TupeloWasm.get();
            try {
                let stateBits = yield tw.getLatest(did);
                let proof = gossip_pb_1.Proof.deserializeBinary(stateBits);
                return new chaintree_1.default({
                    store: clientStore,
                    tip: new cids_1.default(Buffer.from(proof.getTip_asU8())),
                });
            }
            catch (err) {
                throw err;
            }
        });
    }
    Tupelo.getLatest = getLatest;
    // newEmptyTree creates a new ChainTree with the ID populateed in the IBlockService and
    // returns the CID to the tip
    function newEmptyTree(store, publicKey) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("newEmptyTree");
            const tw = yield TupeloWasm.get();
            return tw.newEmptyTree(store, publicKey);
        });
    }
    Tupelo.newEmptyTree = newEmptyTree;
    function tokenPayloadForTransaction(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("tokenPayloadForTransaction");
            const tw = yield TupeloWasm.get();
            const respBits = yield tw.tokenPayloadForTransaction({
                blockService: opts.blockService,
                tip: opts.tip,
                tokenName: opts.tokenName,
                sendId: opts.sendId,
                jsSendTxProof: opts.proof.serializeBinary(),
            });
            return transactions_pb_1.TokenPayload.deserializeBinary(respBits);
        });
    }
    Tupelo.tokenPayloadForTransaction = tokenPayloadForTransaction;
    /**
     * verifyProof takes a proof and makes sure it meets the requirements
     * for being a valid signed proof (2/3 of signers, valid signature, etc)
     * @param proof - the Proof (often returned by a playTransactions)
     * @public
     */
    function verifyProof(proof) {
        return __awaiter(this, void 0, void 0, function* () {
            const tw = yield TupeloWasm.get();
            return tw.verifyProof(proof.serializeBinary());
        });
    }
    Tupelo.verifyProof = verifyProof;
    function startClient(pubsub, group, store) {
        return __awaiter(this, void 0, void 0, function* () {
            const tw = yield TupeloWasm.get();
            clientStore = store;
            return tw.startClient({
                pubsub: pubsub,
                notaryGroup: group.serializeBinary(),
                blockService: store,
            });
        });
    }
    Tupelo.startClient = startClient;
    function playTransactions(tree, transactions) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("playTransactions");
            if (tree.key == undefined) {
                throw new Error("playing transactions on a tree requires the tree to have a private key, use tree.key = <ecdsaKey>");
            }
            const tw = yield TupeloWasm.get();
            let transBits = new Array();
            for (var t of transactions) {
                const serialized = t.serializeBinary();
                transBits = transBits.concat(serialized);
            }
            const privateKey = tree.key.privateKey ? tree.key.privateKey : new Uint8Array();
            if (privateKey.length == 0) {
                throw new Error("can only play transactions on a tree with a private key attached");
            }
            const resp = yield tw.playTransactions({
                privateKey: privateKey,
                tip: tree.tip,
                transactions: transBits,
            });
            const proof = gossip_pb_1.Proof.deserializeBinary(resp);
            tree.tip = new cids_1.default(Buffer.from(proof.getTip_asU8()));
            return proof;
        });
    }
    Tupelo.playTransactions = playTransactions;
    function setLogLevel(name, level) {
        return __awaiter(this, void 0, void 0, function* () {
            const tw = yield TupeloWasm.get();
            return tw.setLogLevel(name, level);
        });
    }
    Tupelo.setLogLevel = setLogLevel;
})(Tupelo = exports.Tupelo || (exports.Tupelo = {}));
exports.default = Tupelo;
